"""The sequence commands module.

These commands are used in the following models:
AWG5K, AWG5KC, AWG7K, AWG7KC

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SEQuence:ELEMent[n]:GOTO:INDex <target>
    - SEQuence:ELEMent[n]:GOTO:INDex?
    - SEQuence:ELEMent[n]:GOTO:STATe <goto_state>
    - SEQuence:ELEMent[n]:GOTO:STATe?
    - SEQuence:ELEMent[n]:JTARget:INDex <target>
    - SEQuence:ELEMent[n]:JTARget:INDex?
    - SEQuence:ELEMent[n]:JTARget:TYPE {INDex|NEXT|OFF}
    - SEQuence:ELEMent[n]:JTARget:TYPE?
    - SEQuence:ELEMent[n]:LOOP:COUNt <NR1>
    - SEQuence:ELEMent[n]:LOOP:COUNt?
    - SEQuence:ELEMent[n]:LOOP:INFinite <loop_state>
    - SEQuence:ELEMent[n]:LOOP:INFinite?
    - SEQuence:ELEMent[n]:SUBSequence <subseq_name>
    - SEQuence:ELEMent[n]:SUBSequence?
    - SEQuence:ELEMent[n]:TWAit <Boolean>
    - SEQuence:ELEMent[n]:TWAit?
    - SEQuence:ELEMent[n]:WAVeform[m] [1|2|3|4] <wfm_name>[1|2|3|4]?
    - SEQuence:JUMP:IMMediate <target>
    - SEQuence:LENGth <NR1>
    - SEQuence:LENGth?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SequenceLength(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:LENGth`` command.

    Description:
        - This command and query sets or returns the sequence length. Use this command to create an
          uninitialized sequence. You can also use the command to clear all sequence elements in a
          single action by passing 0 as the parameter. However, this action cannot be undone so
          exercise necessary caution. Also note that passing a value less than the sequence's
          current length will cause some sequence elements to be deleted at the end of the sequence.
          For example if ``SEQuence:LENGth?`` returns 200 and you subsequently send
          ``SEQuence:LENGth 21``, all sequence elements except the first 20 will be deleted.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:LENGth?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:LENGth?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SEQuence:LENGth value`` command.

    SCPI Syntax:
        ```
        - SEQuence:LENGth <NR1>
        - SEQuence:LENGth?
        ```

    Info:
        - ``<NR1>``
    """


class SequenceJumpImmediate(SCPICmdWrite):
    """The ``SEQuence:JUMP:IMMediate`` command.

    Description:
        - This command forces the sequencer to jump to the specified element index. This is called a
          Force jump. This command does not require an event for executing the jump. Also, the Jump
          target specified for event jump is not used here.

    Usage:
        - Using the ``.write(value)`` method will send the ``SEQuence:JUMP:IMMediate value``
          command.

    SCPI Syntax:
        ```
        - SEQuence:JUMP:IMMediate <target>
        ```

    Info:
        - ``<target>`` ::=<NR1>.
    """


class SequenceJump(SCPICmdRead):
    """The ``SEQuence:JUMP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:JUMP?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:JUMP?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.immediate``: The ``SEQuence:JUMP:IMMediate`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._immediate = SequenceJumpImmediate(device, f"{self._cmd_syntax}:IMMediate")

    @property
    def immediate(self) -> SequenceJumpImmediate:
        """Return the ``SEQuence:JUMP:IMMediate`` command.

        Description:
            - This command forces the sequencer to jump to the specified element index. This is
              called a Force jump. This command does not require an event for executing the jump.
              Also, the Jump target specified for event jump is not used here.

        Usage:
            - Using the ``.write(value)`` method will send the ``SEQuence:JUMP:IMMediate value``
              command.

        SCPI Syntax:
            ```
            - SEQuence:JUMP:IMMediate <target>
            ```

        Info:
            - ``<target>`` ::=<NR1>.
        """
        return self._immediate


class SequenceElementItemWaveformItem(ValidatedDynamicNumberCmd, SCPICmdWrite):
    """The ``SEQuence:ELEMent[n]:WAVeform[m]`` command.

    Description:
        - This command and query sets or returns the waveform for a sequence element.

    Usage:
        - Using the ``.write(value)`` method will send the ``SEQuence:ELEMent[n]:WAVeform[m] value``
          command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:WAVeform[m] [1|2|3|4] <wfm_name>[1|2|3|4]?
        ```

    Info:
        - ``<wfm_name`` >::=<string>.
    """


class SequenceElementItemTwait(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:TWAit`` command.

    Description:
        - This command and query sets or returns the wait trigger state for an element. Send a
          trigger signal in one of the following ways: By using an external trigger signal. By
          pressing the 'Force Trigger' button on the front panel. By sending the ``*TRG`` remote
          command.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:TWAit?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:TWAit?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SEQuence:ELEMent[n]:TWAit value``
          command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:TWAit <Boolean>
        - SEQuence:ELEMent[n]:TWAit?
        ```

    Info:
        - ``<wait_trigger_state>`` ::=<Boolean>.
        - ``0`` indicates OFF.
        - ``1`` indicates ON.
    """


class SequenceElementItemSubsequence(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:SUBSequence`` command.

    Description:
        - This command and query sets or returns the subsequence for a sequence element. The
          AWG5012B, AWG5000C, and AWG7000C (option 09) series instruments support Subsequence. The
          subsequence name can be a null string (''). When a waveform is assigned to this sequence,
          the command returns ''.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:SUBSequence?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:SUBSequence?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SEQuence:ELEMent[n]:SUBSequence value``
          command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:SUBSequence <subseq_name>
        - SEQuence:ELEMent[n]:SUBSequence?
        ```

    Info:
        - ``<subseq_name>`` ::=<string>.
    """


class SequenceElementItemLoopInfinite(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:LOOP:INFinite`` command.

    Description:
        - This command and query sets or returns the infinite looping state for a sequence element.
          When an infinite loop is set on an element, the sequencer continuously executes that
          element. To break the infinite loop, either issue the ``AWGCONTROL:STOP:IMMEDIATE``
          command or change the run mode to Continuous by using ``AWGCONTROL:RMODE`` command.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:LOOP:INFinite?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:LOOP:INFinite?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SEQuence:ELEMent[n]:LOOP:INFinite value`` command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:LOOP:INFinite <loop_state>
        - SEQuence:ELEMent[n]:LOOP:INFinite?
        ```

    Info:
        - ``<loop_state>`` ::=<Boolean>.
        - ``0`` indicates OFF.
        - ``1`` indicates ON.
    """


class SequenceElementItemLoopCount(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:LOOP:COUNt`` command.

    Description:
        - This command and query sets or returns the loop count. Loop count setting for an element
          is ignored if ``SEQUENCE:ELEMENTN:LOOP:INFINITE`` is set to ON.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:LOOP:COUNt?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:LOOP:COUNt?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SEQuence:ELEMent[n]:LOOP:COUNt value``
          command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:LOOP:COUNt <NR1>
        - SEQuence:ELEMent[n]:LOOP:COUNt?
        ```

    Info:
        - ``<NR1>``
    """


class SequenceElementItemLoop(SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:LOOP`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:LOOP?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:LOOP?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.count``: The ``SEQuence:ELEMent[n]:LOOP:COUNt`` command.
        - ``.infinite``: The ``SEQuence:ELEMent[n]:LOOP:INFinite`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._count = SequenceElementItemLoopCount(device, f"{self._cmd_syntax}:COUNt")
        self._infinite = SequenceElementItemLoopInfinite(device, f"{self._cmd_syntax}:INFinite")

    @property
    def count(self) -> SequenceElementItemLoopCount:
        """Return the ``SEQuence:ELEMent[n]:LOOP:COUNt`` command.

        Description:
            - This command and query sets or returns the loop count. Loop count setting for an
              element is ignored if ``SEQUENCE:ELEMENTN:LOOP:INFINITE`` is set to ON.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:LOOP:COUNt?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:LOOP:COUNt?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:LOOP:COUNt value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:LOOP:COUNt <NR1>
            - SEQuence:ELEMent[n]:LOOP:COUNt?
            ```

        Info:
            - ``<NR1>``
        """
        return self._count

    @property
    def infinite(self) -> SequenceElementItemLoopInfinite:
        """Return the ``SEQuence:ELEMent[n]:LOOP:INFinite`` command.

        Description:
            - This command and query sets or returns the infinite looping state for a sequence
              element. When an infinite loop is set on an element, the sequencer continuously
              executes that element. To break the infinite loop, either issue the
              ``AWGCONTROL:STOP:IMMEDIATE`` command or change the run mode to Continuous by using
              ``AWGCONTROL:RMODE`` command.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:LOOP:INFinite?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SEQuence:ELEMent[n]:LOOP:INFinite?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:LOOP:INFinite value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:LOOP:INFinite <loop_state>
            - SEQuence:ELEMent[n]:LOOP:INFinite?
            ```

        Info:
            - ``<loop_state>`` ::=<Boolean>.
            - ``0`` indicates OFF.
            - ``1`` indicates ON.
        """
        return self._infinite


class SequenceElementItemJtargetType(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:JTARget:TYPE`` command.

    Description:
        - This command and query sets or returns the event jump target type for the jump. Generate
          an event in one of the following ways: By connecting an external cable to instrument rear
          panel for external event. By pressing the Force Event button on the front panel. By
          sending the ``EVENT:IMMEDIATE`` remote command.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:JTARget:TYPE?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:JTARget:TYPE?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SEQuence:ELEMent[n]:JTARget:TYPE value`` command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:JTARget:TYPE {INDex|NEXT|OFF}
        - SEQuence:ELEMent[n]:JTARget:TYPE?
        ```

    Info:
        - ``INDex`` . This enables the sequencer to jump to an index set using
          ``SEQuence:ELEMent1:JTARget:INDexcommand``.
        - ``NEXT`` . This enables the sequencer to jump to the next sequence element.
          ``SEQuence:ELEMent1:JTARget:INDexsetting`` is ignored.
        - ``OFF`` . This enables the sequencer to turn off the event jump state. In this state, even
          if the event occurs, the sequencer ignores it.
    """


class SequenceElementItemJtargetIndex(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:JTARget:INDex`` command.

    Description:
        - This command and query sets or returns the target index for the sequencer's event jump
          operation. Note that this will take effect only when ``SEQUENCE:ELEMENTN:JTARGET:TYPE`` is
          set to INDex.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:JTARget:INDex?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:JTARget:INDex?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SEQuence:ELEMent[n]:JTARget:INDex value`` command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:JTARget:INDex <target>
        - SEQuence:ELEMent[n]:JTARget:INDex?
        ```

    Info:
        - ``<target>`` ::=<NR1>.
        - ``<n>`` is an index number of sequence.
    """


class SequenceElementItemJtarget(SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:JTARget`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:JTARget?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:JTARget?`` query
          and raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.index``: The ``SEQuence:ELEMent[n]:JTARget:INDex`` command.
        - ``.type``: The ``SEQuence:ELEMent[n]:JTARget:TYPE`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._index = SequenceElementItemJtargetIndex(device, f"{self._cmd_syntax}:INDex")
        self._type = SequenceElementItemJtargetType(device, f"{self._cmd_syntax}:TYPE")

    @property
    def index(self) -> SequenceElementItemJtargetIndex:
        """Return the ``SEQuence:ELEMent[n]:JTARget:INDex`` command.

        Description:
            - This command and query sets or returns the target index for the sequencer's event jump
              operation. Note that this will take effect only when
              ``SEQUENCE:ELEMENTN:JTARGET:TYPE`` is set to INDex.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:JTARget:INDex?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SEQuence:ELEMent[n]:JTARget:INDex?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:JTARget:INDex value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:JTARget:INDex <target>
            - SEQuence:ELEMent[n]:JTARget:INDex?
            ```

        Info:
            - ``<target>`` ::=<NR1>.
            - ``<n>`` is an index number of sequence.
        """
        return self._index

    @property
    def type(self) -> SequenceElementItemJtargetType:
        """Return the ``SEQuence:ELEMent[n]:JTARget:TYPE`` command.

        Description:
            - This command and query sets or returns the event jump target type for the jump.
              Generate an event in one of the following ways: By connecting an external cable to
              instrument rear panel for external event. By pressing the Force Event button on the
              front panel. By sending the ``EVENT:IMMEDIATE`` remote command.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:JTARget:TYPE?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SEQuence:ELEMent[n]:JTARget:TYPE?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:JTARget:TYPE value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:JTARget:TYPE {INDex|NEXT|OFF}
            - SEQuence:ELEMent[n]:JTARget:TYPE?
            ```

        Info:
            - ``INDex`` . This enables the sequencer to jump to an index set using
              ``SEQuence:ELEMent1:JTARget:INDexcommand``.
            - ``NEXT`` . This enables the sequencer to jump to the next sequence element.
              ``SEQuence:ELEMent1:JTARget:INDexsetting`` is ignored.
            - ``OFF`` . This enables the sequencer to turn off the event jump state. In this state,
              even if the event occurs, the sequencer ignores it.
        """
        return self._type


class SequenceElementItemGotoState(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:GOTO:STATe`` command.

    Description:
        - This command and query sets or returns the GOTO state of the sequencer. For the
          ``SEQUENCE:ELEMENTN:GOTO:INDEX`` command to take effect, the GOTO state must be set to ON.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:GOTO:STATe?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO:STATe?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO:STATe value``
          command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:GOTO:STATe <goto_state>
        - SEQuence:ELEMent[n]:GOTO:STATe?
        ```

    Info:
        - ``<goto_state>`` ::=<Boolean>.
        - ``0`` indicates OFF.
        - ``1`` indicates ON.
    """


class SequenceElementItemGotoIndex(SCPICmdWrite, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:GOTO:INDex`` command.

    Description:
        - This command and query sets or returns the target index for the GOTO command of the
          sequencer. After generating the waveform specified in a sequence element, the sequencer
          jumps to the element specified as GOTO target. This is an unconditional jump. If GOTO
          target is not specified, the sequencer simply moves on to the next element. If the Loop
          Count is Infinite, the GOTO target which is specified in the element is not used. For this
          command to work, the ``SEQUENCE:ELEMENTN:GOTO:STATE`` must be ON and the sequence element
          must exist. Note that the first element of a sequence is taken to be 1 not 0.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:GOTO:INDex?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO:INDex?``
          query and raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO:INDex value``
          command.

    SCPI Syntax:
        ```
        - SEQuence:ELEMent[n]:GOTO:INDex <target>
        - SEQuence:ELEMent[n]:GOTO:INDex?
        ```

    Info:
        - ``<target>`` ::=<NR1>.
    """


class SequenceElementItemGoto(SCPICmdRead):
    """The ``SEQuence:ELEMent[n]:GOTO`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:GOTO?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.index``: The ``SEQuence:ELEMent[n]:GOTO:INDex`` command.
        - ``.state``: The ``SEQuence:ELEMent[n]:GOTO:STATe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._index = SequenceElementItemGotoIndex(device, f"{self._cmd_syntax}:INDex")
        self._state = SequenceElementItemGotoState(device, f"{self._cmd_syntax}:STATe")

    @property
    def index(self) -> SequenceElementItemGotoIndex:
        """Return the ``SEQuence:ELEMent[n]:GOTO:INDex`` command.

        Description:
            - This command and query sets or returns the target index for the GOTO command of the
              sequencer. After generating the waveform specified in a sequence element, the
              sequencer jumps to the element specified as GOTO target. This is an unconditional
              jump. If GOTO target is not specified, the sequencer simply moves on to the next
              element. If the Loop Count is Infinite, the GOTO target which is specified in the
              element is not used. For this command to work, the ``SEQUENCE:ELEMENTN:GOTO:STATE``
              must be ON and the sequence element must exist. Note that the first element of a
              sequence is taken to be 1 not 0.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:GOTO:INDex?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO:INDex?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:GOTO:INDex value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:GOTO:INDex <target>
            - SEQuence:ELEMent[n]:GOTO:INDex?
            ```

        Info:
            - ``<target>`` ::=<NR1>.
        """
        return self._index

    @property
    def state(self) -> SequenceElementItemGotoState:
        """Return the ``SEQuence:ELEMent[n]:GOTO:STATe`` command.

        Description:
            - This command and query sets or returns the GOTO state of the sequencer. For the
              ``SEQUENCE:ELEMENTN:GOTO:INDEX`` command to take effect, the GOTO state must be set to
              ON.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:GOTO:STATe?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO:STATe?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:GOTO:STATe value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:GOTO:STATe <goto_state>
            - SEQuence:ELEMent[n]:GOTO:STATe?
            ```

        Info:
            - ``<goto_state>`` ::=<Boolean>.
            - ``0`` indicates OFF.
            - ``1`` indicates ON.
        """
        return self._state


class SequenceElementItem(ValidatedDynamicNumberCmd, SCPICmdRead):
    """The ``SEQuence:ELEMent[n]`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.goto``: The ``SEQuence:ELEMent[n]:GOTO`` command tree.
        - ``.jtarget``: The ``SEQuence:ELEMent[n]:JTARget`` command tree.
        - ``.loop``: The ``SEQuence:ELEMent[n]:LOOP`` command tree.
        - ``.subsequence``: The ``SEQuence:ELEMent[n]:SUBSequence`` command.
        - ``.twait``: The ``SEQuence:ELEMent[n]:TWAit`` command.
        - ``.waveform``: The ``SEQuence:ELEMent[n]:WAVeform[m]`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._goto = SequenceElementItemGoto(device, f"{self._cmd_syntax}:GOTO")
        self._jtarget = SequenceElementItemJtarget(device, f"{self._cmd_syntax}:JTARget")
        self._loop = SequenceElementItemLoop(device, f"{self._cmd_syntax}:LOOP")
        self._subsequence = SequenceElementItemSubsequence(
            device, f"{self._cmd_syntax}:SUBSequence"
        )
        self._twait = SequenceElementItemTwait(device, f"{self._cmd_syntax}:TWAit")
        self._waveform: Dict[int, SequenceElementItemWaveformItem] = DefaultDictPassKeyToFactory(
            lambda x: SequenceElementItemWaveformItem(device, f"{self._cmd_syntax}:WAVeform{x}")
        )

    @property
    def goto(self) -> SequenceElementItemGoto:
        """Return the ``SEQuence:ELEMent[n]:GOTO`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:GOTO?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:GOTO?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.index``: The ``SEQuence:ELEMent[n]:GOTO:INDex`` command.
            - ``.state``: The ``SEQuence:ELEMent[n]:GOTO:STATe`` command.
        """
        return self._goto

    @property
    def jtarget(self) -> SequenceElementItemJtarget:
        """Return the ``SEQuence:ELEMent[n]:JTARget`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:JTARget?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:JTARget?``
              query and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.index``: The ``SEQuence:ELEMent[n]:JTARget:INDex`` command.
            - ``.type``: The ``SEQuence:ELEMent[n]:JTARget:TYPE`` command.
        """
        return self._jtarget

    @property
    def loop(self) -> SequenceElementItemLoop:
        """Return the ``SEQuence:ELEMent[n]:LOOP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:LOOP?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:LOOP?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.count``: The ``SEQuence:ELEMent[n]:LOOP:COUNt`` command.
            - ``.infinite``: The ``SEQuence:ELEMent[n]:LOOP:INFinite`` command.
        """
        return self._loop

    @property
    def subsequence(self) -> SequenceElementItemSubsequence:
        """Return the ``SEQuence:ELEMent[n]:SUBSequence`` command.

        Description:
            - This command and query sets or returns the subsequence for a sequence element. The
              AWG5012B, AWG5000C, and AWG7000C (option 09) series instruments support Subsequence.
              The subsequence name can be a null string (''). When a waveform is assigned to this
              sequence, the command returns ''.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:SUBSequence?``
              query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:SUBSequence?``
              query and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:SUBSequence value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:SUBSequence <subseq_name>
            - SEQuence:ELEMent[n]:SUBSequence?
            ```

        Info:
            - ``<subseq_name>`` ::=<string>.
        """
        return self._subsequence

    @property
    def twait(self) -> SequenceElementItemTwait:
        """Return the ``SEQuence:ELEMent[n]:TWAit`` command.

        Description:
            - This command and query sets or returns the wait trigger state for an element. Send a
              trigger signal in one of the following ways: By using an external trigger signal. By
              pressing the 'Force Trigger' button on the front panel. By sending the ``*TRG`` remote
              command.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]:TWAit?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]:TWAit?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SEQuence:ELEMent[n]:TWAit value``
              command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:TWAit <Boolean>
            - SEQuence:ELEMent[n]:TWAit?
            ```

        Info:
            - ``<wait_trigger_state>`` ::=<Boolean>.
            - ``0`` indicates OFF.
            - ``1`` indicates ON.
        """
        return self._twait

    @property
    def waveform(self) -> Dict[int, SequenceElementItemWaveformItem]:
        """Return the ``SEQuence:ELEMent[n]:WAVeform[m]`` command.

        Description:
            - This command and query sets or returns the waveform for a sequence element.

        Usage:
            - Using the ``.write(value)`` method will send the
              ``SEQuence:ELEMent[n]:WAVeform[m] value`` command.

        SCPI Syntax:
            ```
            - SEQuence:ELEMent[n]:WAVeform[m] [1|2|3|4] <wfm_name>[1|2|3|4]?
            ```

        Info:
            - ``<wfm_name`` >::=<string>.
        """
        return self._waveform


class Sequence(SCPICmdRead):
    """The ``SEQuence`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SEQuence?`` query.
        - Using the ``.verify(value)`` method will send the ``SEQuence?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.element``: The ``SEQuence:ELEMent[n]`` command tree.
        - ``.jump``: The ``SEQuence:JUMP`` command tree.
        - ``.length``: The ``SEQuence:LENGth`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SEQuence") -> None:
        super().__init__(device, cmd_syntax)
        self._element: Dict[int, SequenceElementItem] = DefaultDictPassKeyToFactory(
            lambda x: SequenceElementItem(device, f"{self._cmd_syntax}:ELEMent{x}")
        )
        self._jump = SequenceJump(device, f"{self._cmd_syntax}:JUMP")
        self._length = SequenceLength(device, f"{self._cmd_syntax}:LENGth")

    @property
    def element(self) -> Dict[int, SequenceElementItem]:
        """Return the ``SEQuence:ELEMent[n]`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:ELEMent[n]?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:ELEMent[n]?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.goto``: The ``SEQuence:ELEMent[n]:GOTO`` command tree.
            - ``.jtarget``: The ``SEQuence:ELEMent[n]:JTARget`` command tree.
            - ``.loop``: The ``SEQuence:ELEMent[n]:LOOP`` command tree.
            - ``.subsequence``: The ``SEQuence:ELEMent[n]:SUBSequence`` command.
            - ``.twait``: The ``SEQuence:ELEMent[n]:TWAit`` command.
            - ``.waveform``: The ``SEQuence:ELEMent[n]:WAVeform[m]`` command.
        """
        return self._element

    @property
    def jump(self) -> SequenceJump:
        """Return the ``SEQuence:JUMP`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:JUMP?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:JUMP?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.immediate``: The ``SEQuence:JUMP:IMMediate`` command.
        """
        return self._jump

    @property
    def length(self) -> SequenceLength:
        """Return the ``SEQuence:LENGth`` command.

        Description:
            - This command and query sets or returns the sequence length. Use this command to create
              an uninitialized sequence. You can also use the command to clear all sequence elements
              in a single action by passing 0 as the parameter. However, this action cannot be
              undone so exercise necessary caution. Also note that passing a value less than the
              sequence's current length will cause some sequence elements to be deleted at the end
              of the sequence. For example if ``SEQuence:LENGth?`` returns 200 and you subsequently
              send ``SEQuence:LENGth 21``, all sequence elements except the first 20 will be
              deleted.

        Usage:
            - Using the ``.query()`` method will send the ``SEQuence:LENGth?`` query.
            - Using the ``.verify(value)`` method will send the ``SEQuence:LENGth?`` query and raise
              an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SEQuence:LENGth value`` command.

        SCPI Syntax:
            ```
            - SEQuence:LENGth <NR1>
            - SEQuence:LENGth?
            ```

        Info:
            - ``<NR1>``
        """
        return self._length



"""The save commands module.

These commands are used in the following models:
DPO2K, DPO2KB, MSO2K, MSO2KB

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Commands and Queries:
    ```
    - SAVe:ASSIgn:FOLder?
    - SAVe:ASSIgn:TYPe {IMAGe|WAVEform|SETUp}
    - SAVe:ASSIgn:TYPe?
    - SAVe:EVENTtable:BUS<x> <file path>
    - SAVe:IMAGe <file path>
    - SAVe:IMAGe:FILEFormat {PNG|BMP|TIFf}
    - SAVe:IMAGe:FILEFormat?
    - SAVe:IMAGe:LAYout {LANdscape|PORTRait}
    - SAVe:IMAGe:LAYout?
    - SAVe:SETUp {<file path>|<NR1>}
    - SAVe:WAVEform [<wfm>,{REF<x>}] | [<wfm>, <QString>] | [ALL, <QString>]
    - SAVe:WAVEform:FILEFormat {INTERNal|SPREADSheet}
    - SAVe:WAVEform:FILEFormat?
    - SAVe:WAVEform:GATIng {NONe|CURSors|SCREEN}
    - SAVe:WAVEform:GATIng?
    - SAVe:WAVEform:SPREADSheet:RESOlution {FULL|REDUced}
    - SAVe:WAVEform:SPREADSheet:RESOlution?
    ```
"""

from typing import Dict, Optional, TYPE_CHECKING

from ..helpers import (
    DefaultDictPassKeyToFactory,
    SCPICmdRead,
    SCPICmdWrite,
    ValidatedDynamicNumberCmd,
)

if TYPE_CHECKING:
    from tm_devices.drivers.pi.pi_device import PIDevice


class SaveWaveformSpreadsheetResolution(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform:SPREADSheet:RESOlution`` command.

    Description:
        - Specifies whether to save the full (LRL) or reduced (thumb) resolution waveform to a CSV
          file.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:SPREADSheet:RESOlution?``
          query.
        - Using the ``.verify(value)`` method will send the
          ``SAVe:WAVEform:SPREADSheet:RESOlution?`` query and raise an AssertionError if the
          returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the
          ``SAVe:WAVEform:SPREADSheet:RESOlution value`` command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform:SPREADSheet:RESOlution {FULL|REDUced}
        - SAVe:WAVEform:SPREADSheet:RESOlution?
        ```

    Info:
        - ``FULL`` specifies that full resolution waveform is saved to a CSV file.
        - ``REDUced`` specifies that reduced resolution waveform is saved to a CSV file.
    """


class SaveWaveformSpreadsheet(SCPICmdRead):
    """The ``SAVe:WAVEform:SPREADSheet`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:SPREADSheet?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:SPREADSheet?`` query and
          raise an AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.resolution``: The ``SAVe:WAVEform:SPREADSheet:RESOlution`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._resolution = SaveWaveformSpreadsheetResolution(
            device, f"{self._cmd_syntax}:RESOlution"
        )

    @property
    def resolution(self) -> SaveWaveformSpreadsheetResolution:
        """Return the ``SAVe:WAVEform:SPREADSheet:RESOlution`` command.

        Description:
            - Specifies whether to save the full (LRL) or reduced (thumb) resolution waveform to a
              CSV file.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:SPREADSheet:RESOlution?``
              query.
            - Using the ``.verify(value)`` method will send the
              ``SAVe:WAVEform:SPREADSheet:RESOlution?`` query and raise an AssertionError if the
              returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the
              ``SAVe:WAVEform:SPREADSheet:RESOlution value`` command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform:SPREADSheet:RESOlution {FULL|REDUced}
            - SAVe:WAVEform:SPREADSheet:RESOlution?
            ```

        Info:
            - ``FULL`` specifies that full resolution waveform is saved to a CSV file.
            - ``REDUced`` specifies that reduced resolution waveform is saved to a CSV file.
        """
        return self._resolution


class SaveWaveformGating(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform:GATIng`` command.

    Description:
        - Specifies whether save waveform operations should save the entire waveform (NONe) or a
          specified portion of the waveform.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:GATIng?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:GATIng?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:GATIng value`` command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform:GATIng {NONe|CURSors|SCREEN}
        - SAVe:WAVEform:GATIng?
        ```

    Info:
        - ``CURSors`` turns on cursors and the gates are the waveform record points at the cursor
          positions.
        - ``NONe`` saves the entire waveform.
        - ``SCREEN`` , if zoom is on, the gates are the start and end waveform record points of the
          zoom (upper) graticule, otherwise the gates are the start and end waveform record points
          of the main graticule.
    """


class SaveWaveformFileformat(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform:FILEFormat`` command.

    Description:
        - This command specifies the file format to be used when saving waveforms - either an
          internal format, .ISF, or an external comma-delimited spreadsheet format, .CSV, that
          includes waveform header and timing information.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FILEFormat?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:FILEFormat value``
          command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform:FILEFormat {INTERNal|SPREADSheet}
        - SAVe:WAVEform:FILEFormat?
        ```

    Info:
        - ``INTERNal`` specifies to save waveforms using an internal format. The file name should be
          specified with .ISF filename extension. These files can be recalled as reference
          waveforms.
        - ``SPREADSheet`` specifies that waveform data is saved in a format that contains comma
          delimited values. These waveform data files should be named using the .CSV filename
          extension. Saving waveforms in .CSV format enables spreadsheet programs to import the
          data.
    """


class SaveWaveform(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:WAVEform`` command.

    Description:
        - This command saves a specified waveform or all displayed waveforms (excluding serial bus
          waveforms).

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:WAVEform value`` command.

    SCPI Syntax:
        ```
        - SAVe:WAVEform [<wfm>,{REF<x>}] | [<wfm>, <QString>] | [ALL, <QString>]
        ```

    Info:
        - ``<wfm>, <REF<x>>`` saves the specified waveform to the specified reference memory
          location.
        - ``<wfm>, <QString>`` saves the specified waveform to the file specified in the quoted
          string argument.
        - ``ALL, <QString>`` saves all displayed waveforms, excluding serial bus waveforms, to a
          single CSV file specified by the quoted string argument when the
          ``SAVE:WAVEFORM:FILEFORMAT`` is set to SPREADSHEET, or saves all displayed waveforms,
          excluding serial bus waveforms to individual ISF (internal save format) files with a file
          name prefix specified by the argument with an underscore (_) and the waveform ID (such as
          CH1, REF1, MATH) appended to the file name(s).

    Properties:
        - ``.fileformat``: The ``SAVe:WAVEform:FILEFormat`` command.
        - ``.gating``: The ``SAVe:WAVEform:GATIng`` command.
        - ``.spreadsheet``: The ``SAVe:WAVEform:SPREADSheet`` command tree.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._fileformat = SaveWaveformFileformat(device, f"{self._cmd_syntax}:FILEFormat")
        self._gating = SaveWaveformGating(device, f"{self._cmd_syntax}:GATIng")
        self._spreadsheet = SaveWaveformSpreadsheet(device, f"{self._cmd_syntax}:SPREADSheet")

    @property
    def fileformat(self) -> SaveWaveformFileformat:
        """Return the ``SAVe:WAVEform:FILEFormat`` command.

        Description:
            - This command specifies the file format to be used when saving waveforms - either an
              internal format, .ISF, or an external comma-delimited spreadsheet format, .CSV, that
              includes waveform header and timing information.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:FILEFormat?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:FILEFormat?`` query
              and raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:FILEFormat value``
              command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform:FILEFormat {INTERNal|SPREADSheet}
            - SAVe:WAVEform:FILEFormat?
            ```

        Info:
            - ``INTERNal`` specifies to save waveforms using an internal format. The file name
              should be specified with .ISF filename extension. These files can be recalled as
              reference waveforms.
            - ``SPREADSheet`` specifies that waveform data is saved in a format that contains comma
              delimited values. These waveform data files should be named using the .CSV filename
              extension. Saving waveforms in .CSV format enables spreadsheet programs to import the
              data.
        """
        return self._fileformat

    @property
    def gating(self) -> SaveWaveformGating:
        """Return the ``SAVe:WAVEform:GATIng`` command.

        Description:
            - Specifies whether save waveform operations should save the entire waveform (NONe) or a
              specified portion of the waveform.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:GATIng?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:GATIng?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform:GATIng value``
              command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform:GATIng {NONe|CURSors|SCREEN}
            - SAVe:WAVEform:GATIng?
            ```

        Info:
            - ``CURSors`` turns on cursors and the gates are the waveform record points at the
              cursor positions.
            - ``NONe`` saves the entire waveform.
            - ``SCREEN`` , if zoom is on, the gates are the start and end waveform record points of
              the zoom (upper) graticule, otherwise the gates are the start and end waveform record
              points of the main graticule.
        """
        return self._gating

    @property
    def spreadsheet(self) -> SaveWaveformSpreadsheet:
        """Return the ``SAVe:WAVEform:SPREADSheet`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:WAVEform:SPREADSheet?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:WAVEform:SPREADSheet?`` query
              and raise an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.resolution``: The ``SAVe:WAVEform:SPREADSheet:RESOlution`` command.
        """
        return self._spreadsheet


class SaveSetup(SCPICmdWrite):
    """The ``SAVe:SETUp`` command.

    Description:
        - Stores the state of the oscilloscope to a specified memory location. You can later use the
          ``*RCL`` command to restore the oscilloscope to this saved state.

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:SETUp value`` command.

    SCPI Syntax:
        ```
        - SAVe:SETUp {<file path>|<NR1>}
        ```

    Info:
        - ``<file path>`` is the target location for storing the setup file. <file path> is a quoted
          string that defines the file name and path. Input the file path using the form
          ``<drive>:<dir>``/<filename>. <extension> and one or <dir>s are optional. If you do not
          specify them, the oscilloscope will store the file in the current working directory.
          <filename> stands for a filename. (Use of wildcard characters in filenames is not
          supported.) Filename extensions are not required but are highly recommended. For setups,
          use the extension '.SET'.
        - ``<NR1>`` specifies a location for saving the current front-panel setup. The front-panel
          setup value ranges from 1 to 10. Using an out-of-range value causes an execution error.
          Any settings that have been stored previously at this location will be overwritten.
    """


class SaveImageLayout(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:IMAGe:LAYout`` command.

    Description:
        - This command specifies the layout to use for saved screen images.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:IMAGe:LAYout?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:LAYout?`` query and raise
          an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:LAYout value`` command.

    SCPI Syntax:
        ```
        - SAVe:IMAGe:LAYout {LANdscape|PORTRait}
        - SAVe:IMAGe:LAYout?
        ```

    Info:
        - ``LANdscape`` specifies that screen images are saved in landscape format.
        - ``PORTRait`` specifies that screen images are saved in portrait format.
    """


class SaveImageFileformat(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:IMAGe:FILEFormat`` command.

    Description:
        - This command specifies the file format to use for saving screen images.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:IMAGe:FILEFormat?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:FILEFormat?`` query and
          raise an AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:FILEFormat value`` command.

    SCPI Syntax:
        ```
        - SAVe:IMAGe:FILEFormat {PNG|BMP|TIFf}
        - SAVe:IMAGe:FILEFormat?
        ```

    Info:
        - ``PNG`` saves the file in Portable Network Graphics format.
        - ``BMP`` saves the file in Microsoft Windows bitmap format.
        - ``TIFf`` saves the file in Tagged Image File Format.
    """


class SaveImage(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:IMAGe`` command.

    Description:
        - Saves a capture of the screen image to the specified file. Supported image formats are
          PNG, Windows Bitmap, and TIFF. The file format is specified by the
          ``SAVE:IMAGE:FILEFORMAT`` command. The ``SAVE:IMAGE:INKSAVER`` command determines whether
          the data is saved in InkSaver mode.

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:IMAGe value`` command.

    SCPI Syntax:
        ```
        - SAVe:IMAGe <file path>
        ```

    Info:
        - ``<file path>`` is a filename, including path, where the image will be saved. If you do
          not specify a directory, the oscilloscope will store the file in the current working
          directory. File name extensions are not required but are highly recommended.

    Properties:
        - ``.fileformat``: The ``SAVe:IMAGe:FILEFormat`` command.
        - ``.layout``: The ``SAVe:IMAGe:LAYout`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._fileformat = SaveImageFileformat(device, f"{self._cmd_syntax}:FILEFormat")
        self._layout = SaveImageLayout(device, f"{self._cmd_syntax}:LAYout")

    @property
    def fileformat(self) -> SaveImageFileformat:
        """Return the ``SAVe:IMAGe:FILEFormat`` command.

        Description:
            - This command specifies the file format to use for saving screen images.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:IMAGe:FILEFormat?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:FILEFormat?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:FILEFormat value``
              command.

        SCPI Syntax:
            ```
            - SAVe:IMAGe:FILEFormat {PNG|BMP|TIFf}
            - SAVe:IMAGe:FILEFormat?
            ```

        Info:
            - ``PNG`` saves the file in Portable Network Graphics format.
            - ``BMP`` saves the file in Microsoft Windows bitmap format.
            - ``TIFf`` saves the file in Tagged Image File Format.
        """
        return self._fileformat

    @property
    def layout(self) -> SaveImageLayout:
        """Return the ``SAVe:IMAGe:LAYout`` command.

        Description:
            - This command specifies the layout to use for saved screen images.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:IMAGe:LAYout?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:IMAGe:LAYout?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:IMAGe:LAYout value`` command.

        SCPI Syntax:
            ```
            - SAVe:IMAGe:LAYout {LANdscape|PORTRait}
            - SAVe:IMAGe:LAYout?
            ```

        Info:
            - ``LANdscape`` specifies that screen images are saved in landscape format.
            - ``PORTRait`` specifies that screen images are saved in portrait format.
        """
        return self._layout


class SaveEventtableBusItem(ValidatedDynamicNumberCmd, SCPICmdWrite):
    """The ``SAVe:EVENTtable:BUS<x>`` command.

    Description:
        - Saves the data from bus<x> to a specified file and location; where x is the bus number

    Usage:
        - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:BUS<x> value`` command.

    SCPI Syntax:
        ```
        - SAVe:EVENTtable:BUS<x> <file path>
        ```

    Info:
        - ``<file path>`` is a quoted string that defines the file name and path location where the
          event table will be stored.
    """


class SaveEventtable(SCPICmdRead):
    """The ``SAVe:EVENTtable`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:EVENTtable?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:EVENTtable?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.bus``: The ``SAVe:EVENTtable:BUS<x>`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._bus: Dict[int, SaveEventtableBusItem] = DefaultDictPassKeyToFactory(
            lambda x: SaveEventtableBusItem(device, f"{self._cmd_syntax}:BUS{x}")
        )

    @property
    def bus(self) -> Dict[int, SaveEventtableBusItem]:
        """Return the ``SAVe:EVENTtable:BUS<x>`` command.

        Description:
            - Saves the data from bus<x> to a specified file and location; where x is the bus number

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:EVENTtable:BUS<x> value``
              command.

        SCPI Syntax:
            ```
            - SAVe:EVENTtable:BUS<x> <file path>
            ```

        Info:
            - ``<file path>`` is a quoted string that defines the file name and path location where
              the event table will be stored.
        """
        return self._bus


class SaveAssignType(SCPICmdWrite, SCPICmdRead):
    """The ``SAVe:ASSIgn:TYPe`` command.

    Description:
        - This command specifies the assignment of the data to be saved when the front-panel Save
          button is pressed.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:ASSIgn:TYPe?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn:TYPe?`` query and raise an
          AssertionError if the returned value does not match ``value``.
        - Using the ``.write(value)`` method will send the ``SAVe:ASSIgn:TYPe value`` command.

    SCPI Syntax:
        ```
        - SAVe:ASSIgn:TYPe {IMAGe|WAVEform|SETUp}
        - SAVe:ASSIgn:TYPe?
        ```

    Info:
        - ``IMAGe`` assigns the Save button to save screen images.
        - ``WAVEform`` assigns the Save button to save waveforms.
        - ``SETUp`` assigns the Save button to save setups.
    """


class SaveAssignFolder(SCPICmdRead):
    """The ``SAVe:ASSIgn:FOLder`` command.

    Description:
        - Returns the file name that the next front-panel Save button operation will save the
          requested save type data to.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:ASSIgn:FOLder?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn:FOLder?`` query and raise
          an AssertionError if the returned value does not match ``value``.

    SCPI Syntax:
        ```
        - SAVe:ASSIgn:FOLder?
        ```
    """


class SaveAssign(SCPICmdRead):
    """The ``SAVe:ASSIgn`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe:ASSIgn?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.folder``: The ``SAVe:ASSIgn:FOLder`` command.
        - ``.type``: The ``SAVe:ASSIgn:TYPe`` command.
    """

    def __init__(self, device: Optional["PIDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._folder = SaveAssignFolder(device, f"{self._cmd_syntax}:FOLder")
        self._type = SaveAssignType(device, f"{self._cmd_syntax}:TYPe")

    @property
    def folder(self) -> SaveAssignFolder:
        """Return the ``SAVe:ASSIgn:FOLder`` command.

        Description:
            - Returns the file name that the next front-panel Save button operation will save the
              requested save type data to.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:ASSIgn:FOLder?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn:FOLder?`` query and
              raise an AssertionError if the returned value does not match ``value``.

        SCPI Syntax:
            ```
            - SAVe:ASSIgn:FOLder?
            ```
        """
        return self._folder

    @property
    def type(self) -> SaveAssignType:
        """Return the ``SAVe:ASSIgn:TYPe`` command.

        Description:
            - This command specifies the assignment of the data to be saved when the front-panel
              Save button is pressed.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:ASSIgn:TYPe?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn:TYPe?`` query and
              raise an AssertionError if the returned value does not match ``value``.
            - Using the ``.write(value)`` method will send the ``SAVe:ASSIgn:TYPe value`` command.

        SCPI Syntax:
            ```
            - SAVe:ASSIgn:TYPe {IMAGe|WAVEform|SETUp}
            - SAVe:ASSIgn:TYPe?
            ```

        Info:
            - ``IMAGe`` assigns the Save button to save screen images.
            - ``WAVEform`` assigns the Save button to save waveforms.
            - ``SETUp`` assigns the Save button to save setups.
        """
        return self._type


class Save(SCPICmdRead):
    """The ``SAVe`` command tree.

    Usage:
        - Using the ``.query()`` method will send the ``SAVe?`` query.
        - Using the ``.verify(value)`` method will send the ``SAVe?`` query and raise an
          AssertionError if the returned value does not match ``value``.

    Properties:
        - ``.assign``: The ``SAVe:ASSIgn`` command tree.
        - ``.eventtable``: The ``SAVe:EVENTtable`` command tree.
        - ``.image``: The ``SAVe:IMAGe`` command.
        - ``.setup``: The ``SAVe:SETUp`` command.
        - ``.waveform``: The ``SAVe:WAVEform`` command.
    """

    def __init__(self, device: Optional["PIDevice"] = None, cmd_syntax: str = "SAVe") -> None:
        super().__init__(device, cmd_syntax)
        self._assign = SaveAssign(device, f"{self._cmd_syntax}:ASSIgn")
        self._eventtable = SaveEventtable(device, f"{self._cmd_syntax}:EVENTtable")
        self._image = SaveImage(device, f"{self._cmd_syntax}:IMAGe")
        self._setup = SaveSetup(device, f"{self._cmd_syntax}:SETUp")
        self._waveform = SaveWaveform(device, f"{self._cmd_syntax}:WAVEform")

    @property
    def assign(self) -> SaveAssign:
        """Return the ``SAVe:ASSIgn`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:ASSIgn?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:ASSIgn?`` query and raise an
              AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.folder``: The ``SAVe:ASSIgn:FOLder`` command.
            - ``.type``: The ``SAVe:ASSIgn:TYPe`` command.
        """
        return self._assign

    @property
    def eventtable(self) -> SaveEventtable:
        """Return the ``SAVe:EVENTtable`` command tree.

        Usage:
            - Using the ``.query()`` method will send the ``SAVe:EVENTtable?`` query.
            - Using the ``.verify(value)`` method will send the ``SAVe:EVENTtable?`` query and raise
              an AssertionError if the returned value does not match ``value``.

        Sub-properties:
            - ``.bus``: The ``SAVe:EVENTtable:BUS<x>`` command.
        """
        return self._eventtable

    @property
    def image(self) -> SaveImage:
        """Return the ``SAVe:IMAGe`` command.

        Description:
            - Saves a capture of the screen image to the specified file. Supported image formats are
              PNG, Windows Bitmap, and TIFF. The file format is specified by the
              ``SAVE:IMAGE:FILEFORMAT`` command. The ``SAVE:IMAGE:INKSAVER`` command determines
              whether the data is saved in InkSaver mode.

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:IMAGe value`` command.

        SCPI Syntax:
            ```
            - SAVe:IMAGe <file path>
            ```

        Info:
            - ``<file path>`` is a filename, including path, where the image will be saved. If you
              do not specify a directory, the oscilloscope will store the file in the current
              working directory. File name extensions are not required but are highly recommended.

        Sub-properties:
            - ``.fileformat``: The ``SAVe:IMAGe:FILEFormat`` command.
            - ``.layout``: The ``SAVe:IMAGe:LAYout`` command.
        """
        return self._image

    @property
    def setup(self) -> SaveSetup:
        """Return the ``SAVe:SETUp`` command.

        Description:
            - Stores the state of the oscilloscope to a specified memory location. You can later use
              the ``*RCL`` command to restore the oscilloscope to this saved state.

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:SETUp value`` command.

        SCPI Syntax:
            ```
            - SAVe:SETUp {<file path>|<NR1>}
            ```

        Info:
            - ``<file path>`` is the target location for storing the setup file. <file path> is a
              quoted string that defines the file name and path. Input the file path using the form
              ``<drive>:<dir>``/<filename>. <extension> and one or <dir>s are optional. If you do
              not specify them, the oscilloscope will store the file in the current working
              directory. <filename> stands for a filename. (Use of wildcard characters in filenames
              is not supported.) Filename extensions are not required but are highly recommended.
              For setups, use the extension '.SET'.
            - ``<NR1>`` specifies a location for saving the current front-panel setup. The
              front-panel setup value ranges from 1 to 10. Using an out-of-range value causes an
              execution error. Any settings that have been stored previously at this location will
              be overwritten.
        """
        return self._setup

    @property
    def waveform(self) -> SaveWaveform:
        """Return the ``SAVe:WAVEform`` command.

        Description:
            - This command saves a specified waveform or all displayed waveforms (excluding serial
              bus waveforms).

        Usage:
            - Using the ``.write(value)`` method will send the ``SAVe:WAVEform value`` command.

        SCPI Syntax:
            ```
            - SAVe:WAVEform [<wfm>,{REF<x>}] | [<wfm>, <QString>] | [ALL, <QString>]
            ```

        Info:
            - ``<wfm>, <REF<x>>`` saves the specified waveform to the specified reference memory
              location.
            - ``<wfm>, <QString>`` saves the specified waveform to the file specified in the quoted
              string argument.
            - ``ALL, <QString>`` saves all displayed waveforms, excluding serial bus waveforms, to a
              single CSV file specified by the quoted string argument when the
              ``SAVE:WAVEFORM:FILEFORMAT`` is set to SPREADSHEET, or saves all displayed waveforms,
              excluding serial bus waveforms to individual ISF (internal save format) files with a
              file name prefix specified by the argument with an underscore (_) and the waveform ID
              (such as CH1, REF1, MATH) appended to the file name(s).

        Sub-properties:
            - ``.fileformat``: The ``SAVe:WAVEform:FILEFormat`` command.
            - ``.gating``: The ``SAVe:WAVEform:GATIng`` command.
            - ``.spreadsheet``: The ``SAVe:WAVEform:SPREADSheet`` command tree.
        """
        return self._waveform



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The comm commands module.

These commands are used in the following models:
SS3706A

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - comm.gpib.enable
    - comm.lan.enable
    - comm.lan.rawsockets.enable
    - comm.lan.telnet.enable
    - comm.lan.vxi11.enable
    - comm.lan.web.enable
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class CommLanWeb(BaseTSPCmd):
    """The ``comm.lan.web`` command tree.

    Properties and methods:
        - ``.enable``: The ``comm.lan.web.enable`` attribute.
    """

    @property
    def enable(self) -> str:
        """Access the ``comm.lan.web.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using the web interface is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.web.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.web.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.web.enable = value
            - print(comm.lan.web.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``comm.lan.web.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using the web interface is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.web.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.web.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.web.enable = value
            - print(comm.lan.web.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class CommLanVxi11(BaseTSPCmd):
    """The ``comm.lan.vxi11`` command tree.

    Properties and methods:
        - ``.enable``: The ``comm.lan.vxi11.enable`` attribute.
    """

    @property
    def enable(self) -> str:
        """Access the ``comm.lan.vxi11.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using a VXI-11 connection is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.vxi11.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.vxi11.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.vxi11.enable = value
            - print(comm.lan.vxi11.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``comm.lan.vxi11.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using a VXI-11 connection is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.vxi11.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.vxi11.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.vxi11.enable = value
            - print(comm.lan.vxi11.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class CommLanTelnet(BaseTSPCmd):
    """The ``comm.lan.telnet`` command tree.

    Properties and methods:
        - ``.enable``: The ``comm.lan.telnet.enable`` attribute.
    """

    @property
    def enable(self) -> str:
        """Access the ``comm.lan.telnet.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using Telnet is enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.telnet.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.telnet.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.telnet.enable = value
            - print(comm.lan.telnet.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``comm.lan.telnet.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using Telnet is enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.telnet.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.telnet.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.telnet.enable = value
            - print(comm.lan.telnet.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class CommLanRawsockets(BaseTSPCmd):
    """The ``comm.lan.rawsockets`` command tree.

    Properties and methods:
        - ``.enable``: The ``comm.lan.rawsockets.enable`` attribute.
    """

    @property
    def enable(self) -> str:
        """Access the ``comm.lan.rawsockets.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using raw socket is enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.rawsockets.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.rawsockets.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.rawsockets.enable = value
            - print(comm.lan.rawsockets.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``comm.lan.rawsockets.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using raw socket is enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.rawsockets.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.rawsockets.enable = value``
              command.

        TSP Syntax:
            ```
            - comm.lan.rawsockets.enable = value
            - print(comm.lan.rawsockets.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class CommLan(BaseTSPCmd):
    """The ``comm.lan`` command tree.

    Properties and methods:
        - ``.enable``: The ``comm.lan.enable`` attribute.
        - ``.rawsockets``: The ``comm.lan.rawsockets`` command tree.
        - ``.telnet``: The ``comm.lan.telnet`` command tree.
        - ``.vxi11``: The ``comm.lan.vxi11`` command tree.
        - ``.web``: The ``comm.lan.web`` command tree.
    """

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._rawsockets = CommLanRawsockets(device, f"{self._cmd_syntax}.rawsockets")
        self._telnet = CommLanTelnet(device, f"{self._cmd_syntax}.telnet")
        self._vxi11 = CommLanVxi11(device, f"{self._cmd_syntax}.vxi11")
        self._web = CommLanWeb(device, f"{self._cmd_syntax}.web")

    @property
    def enable(self) -> str:
        """Access the ``comm.lan.enable`` attribute.

        Description:
            - This attribute controls whether or not any communication using the LAN connection is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.enable = value`` command.

        TSP Syntax:
            ```
            - comm.lan.enable = value
            - print(comm.lan.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``comm.lan.enable`` attribute.

        Description:
            - This attribute controls whether or not any communication using the LAN connection is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.lan.enable)`` query.
            - Setting this property to a value will send the ``comm.lan.enable = value`` command.

        TSP Syntax:
            ```
            - comm.lan.enable = value
            - print(comm.lan.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def rawsockets(self) -> CommLanRawsockets:
        """Return the ``comm.lan.rawsockets`` command tree.

        Sub-properties and sub-methods:
            - ``.enable``: The ``comm.lan.rawsockets.enable`` attribute.
        """
        return self._rawsockets

    @property
    def telnet(self) -> CommLanTelnet:
        """Return the ``comm.lan.telnet`` command tree.

        Sub-properties and sub-methods:
            - ``.enable``: The ``comm.lan.telnet.enable`` attribute.
        """
        return self._telnet

    @property
    def vxi11(self) -> CommLanVxi11:
        """Return the ``comm.lan.vxi11`` command tree.

        Sub-properties and sub-methods:
            - ``.enable``: The ``comm.lan.vxi11.enable`` attribute.
        """
        return self._vxi11

    @property
    def web(self) -> CommLanWeb:
        """Return the ``comm.lan.web`` command tree.

        Sub-properties and sub-methods:
            - ``.enable``: The ``comm.lan.web.enable`` attribute.
        """
        return self._web


class CommGpib(BaseTSPCmd):
    """The ``comm.gpib`` command tree.

    Properties and methods:
        - ``.enable``: The ``comm.gpib.enable`` attribute.
    """

    @property
    def enable(self) -> str:
        """Access the ``comm.gpib.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using the GPIB connection is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.gpib.enable)`` query.
            - Setting this property to a value will send the ``comm.gpib.enable = value`` command.

        TSP Syntax:
            ```
            - comm.gpib.enable = value
            - print(comm.gpib.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".enable"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.enable)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @enable.setter
    def enable(self, value: Union[str, float]) -> None:
        """Access the ``comm.gpib.enable`` attribute.

        Description:
            - This attribute describes whether or not communication using the GPIB connection is
              enabled.

        Usage:
            - Accessing this property will send the ``print(comm.gpib.enable)`` query.
            - Setting this property to a value will send the ``comm.gpib.enable = value`` command.

        TSP Syntax:
            ```
            - comm.gpib.enable = value
            - print(comm.gpib.enable)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".enable", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.enable = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.enable`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Comm(BaseTSPCmd):
    """The ``comm`` command tree.

    Properties and methods:
        - ``.gpib``: The ``comm.gpib`` command tree.
        - ``.lan``: The ``comm.lan`` command tree.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "comm") -> None:
        super().__init__(device, cmd_syntax)
        self._gpib = CommGpib(device, f"{self._cmd_syntax}.gpib")
        self._lan = CommLan(device, f"{self._cmd_syntax}.lan")

    @property
    def gpib(self) -> CommGpib:
        """Return the ``comm.gpib`` command tree.

        Sub-properties and sub-methods:
            - ``.enable``: The ``comm.gpib.enable`` attribute.
        """
        return self._gpib

    @property
    def lan(self) -> CommLan:
        """Return the ``comm.lan`` command tree.

        Sub-properties and sub-methods:
            - ``.enable``: The ``comm.lan.enable`` attribute.
            - ``.rawsockets``: The ``comm.lan.rawsockets`` command tree.
            - ``.telnet``: The ``comm.lan.telnet`` command tree.
            - ``.vxi11``: The ``comm.lan.vxi11`` command tree.
            - ``.web``: The ``comm.lan.web`` command tree.
        """
        return self._lan



# ruff: noqa: D402,PLR0913
# pyright: reportConstantRedefinition=none
"""The scan commands module.

These commands are used in the following models:
DMM6500

THIS FILE IS AUTO-GENERATED, IT SHOULD NOT BE MANUALLY MODIFIED.

Please report an issue if one is found.

Attributes and Functions:
    ```
    - scan.add()
    - scan.addsinglestep()
    - scan.buffer
    - scan.bypass
    - scan.channel.stimulus
    - scan.create()
    - scan.export()
    - scan.learnlimits()
    - scan.list()
    - scan.measure.interval
    - scan.measure.stimulus
    - scan.mode
    - scan.monitor.channel
    - scan.monitor.limit.high.value
    - scan.monitor.limit.low.value
    - scan.monitor.mode
    - scan.restart
    - scan.scancount
    - scan.scaninterval
    - scan.start.stimulus
    - scan.stepcount
    ```
"""

from typing import Optional, TYPE_CHECKING, Union

from ..helpers import BaseTSPCmd, NoDeviceProvidedError

if TYPE_CHECKING:
    from tm_devices.drivers.pi.tsp_device import TSPDevice


class ScanStart(BaseTSPCmd):
    """The ``scan.start`` command tree.

    Properties and methods:
        - ``.stimulus``: The ``scan.start.stimulus`` attribute.
    """

    @property
    def stimulus(self) -> str:
        """Access the ``scan.start.stimulus`` attribute.

        Description:
            - This attribute determines which event starts the scan.

        Usage:
            - Accessing this property will send the ``print(scan.start.stimulus)`` query.
            - Setting this property to a value will send the ``scan.start.stimulus = value``
              command.

        TSP Syntax:
            ```
            - scan.start.stimulus = value
            - print(scan.start.stimulus)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``scan.start.stimulus`` attribute.

        Description:
            - This attribute determines which event starts the scan.

        Usage:
            - Accessing this property will send the ``print(scan.start.stimulus)`` query.
            - Setting this property to a value will send the ``scan.start.stimulus = value``
              command.

        TSP Syntax:
            ```
            - scan.start.stimulus = value
            - print(scan.start.stimulus)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanMonitorLimitLow(BaseTSPCmd):
    """The ``scan.monitor.limit.low`` command tree.

    Properties and methods:
        - ``.value``: The ``scan.monitor.limit.low.value`` attribute.
    """

    @property
    def value(self) -> str:
        """Access the ``scan.monitor.limit.low.value`` attribute.

        Description:
            - This attribute defines the low limit to be used by the scan monitor.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.limit.low.value)`` query.
            - Setting this property to a value will send the
              ``scan.monitor.limit.low.value = value`` command.

        TSP Syntax:
            ```
            - scan.monitor.limit.low.value = value
            - print(scan.monitor.limit.low.value)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".value"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.value)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.value`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @value.setter
    def value(self, value: Union[str, float]) -> None:
        """Access the ``scan.monitor.limit.low.value`` attribute.

        Description:
            - This attribute defines the low limit to be used by the scan monitor.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.limit.low.value)`` query.
            - Setting this property to a value will send the
              ``scan.monitor.limit.low.value = value`` command.

        TSP Syntax:
            ```
            - scan.monitor.limit.low.value = value
            - print(scan.monitor.limit.low.value)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".value", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.value = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.value`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanMonitorLimitHigh(BaseTSPCmd):
    """The ``scan.monitor.limit.high`` command tree.

    Properties and methods:
        - ``.value``: The ``scan.monitor.limit.high.value`` attribute.
    """

    @property
    def value(self) -> str:
        """Access the ``scan.monitor.limit.high.value`` attribute.

        Description:
            - This attribute specifies the high limit to be used by the scan monitor.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.limit.high.value)`` query.
            - Setting this property to a value will send the
              ``scan.monitor.limit.high.value = value`` command.

        TSP Syntax:
            ```
            - scan.monitor.limit.high.value = value
            - print(scan.monitor.limit.high.value)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".value"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.value)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.value`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @value.setter
    def value(self, value: Union[str, float]) -> None:
        """Access the ``scan.monitor.limit.high.value`` attribute.

        Description:
            - This attribute specifies the high limit to be used by the scan monitor.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.limit.high.value)`` query.
            - Setting this property to a value will send the
              ``scan.monitor.limit.high.value = value`` command.

        TSP Syntax:
            ```
            - scan.monitor.limit.high.value = value
            - print(scan.monitor.limit.high.value)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".value", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.value = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.value`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanMonitorLimit(BaseTSPCmd):
    """The ``scan.monitor.limit`` command tree.

    Properties and methods:
        - ``.high``: The ``scan.monitor.limit.high`` command tree.
        - ``.low``: The ``scan.monitor.limit.low`` command tree.
    """

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._high = ScanMonitorLimitHigh(device, f"{self._cmd_syntax}.high")
        self._low = ScanMonitorLimitLow(device, f"{self._cmd_syntax}.low")

    @property
    def high(self) -> ScanMonitorLimitHigh:
        """Return the ``scan.monitor.limit.high`` command tree.

        Sub-properties and sub-methods:
            - ``.value``: The ``scan.monitor.limit.high.value`` attribute.
        """
        return self._high

    @property
    def low(self) -> ScanMonitorLimitLow:
        """Return the ``scan.monitor.limit.low`` command tree.

        Sub-properties and sub-methods:
            - ``.value``: The ``scan.monitor.limit.low.value`` attribute.
        """
        return self._low


class ScanMonitor(BaseTSPCmd):
    """The ``scan.monitor`` command tree.

    Properties and methods:
        - ``.channel``: The ``scan.monitor.channel`` attribute.
        - ``.limit``: The ``scan.monitor.limit`` command tree.
        - ``.mode``: The ``scan.monitor.mode`` attribute.
    """

    def __init__(self, device: Optional["TSPDevice"], cmd_syntax: str) -> None:
        super().__init__(device, cmd_syntax)
        self._limit = ScanMonitorLimit(device, f"{self._cmd_syntax}.limit")

    @property
    def channel(self) -> str:
        """Access the ``scan.monitor.channel`` attribute.

        Description:
            - This attribute defines which channel to monitor for a limit to be reached before
              starting the scan.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.channel)`` query.
            - Setting this property to a value will send the ``scan.monitor.channel = value``
              command.

        TSP Syntax:
            ```
            - scan.monitor.channel = value
            - print(scan.monitor.channel)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".channel"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.channel)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.channel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @channel.setter
    def channel(self, value: Union[str, float]) -> None:
        """Access the ``scan.monitor.channel`` attribute.

        Description:
            - This attribute defines which channel to monitor for a limit to be reached before
              starting the scan.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.channel)`` query.
            - Setting this property to a value will send the ``scan.monitor.channel = value``
              command.

        TSP Syntax:
            ```
            - scan.monitor.channel = value
            - print(scan.monitor.channel)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".channel", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.channel = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.channel`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def limit(self) -> ScanMonitorLimit:
        """Return the ``scan.monitor.limit`` command tree.

        Sub-properties and sub-methods:
            - ``.high``: The ``scan.monitor.limit.high`` command tree.
            - ``.low``: The ``scan.monitor.limit.low`` command tree.
        """
        return self._limit

    @property
    def mode(self) -> str:
        """Access the ``scan.monitor.mode`` attribute.

        Description:
            - This attribute determines if a scan starts immediately when triggered or after
              measurements reach a set value.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.mode)`` query.
            - Setting this property to a value will send the ``scan.monitor.mode = value`` command.

        TSP Syntax:
            ```
            - scan.monitor.mode = value
            - print(scan.monitor.mode)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.mode)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``scan.monitor.mode`` attribute.

        Description:
            - This attribute determines if a scan starts immediately when triggered or after
              measurements reach a set value.

        Usage:
            - Accessing this property will send the ``print(scan.monitor.mode)`` query.
            - Setting this property to a value will send the ``scan.monitor.mode = value`` command.

        TSP Syntax:
            ```
            - scan.monitor.mode = value
            - print(scan.monitor.mode)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.mode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanMeasure(BaseTSPCmd):
    """The ``scan.measure`` command tree.

    Properties and methods:
        - ``.interval``: The ``scan.measure.interval`` attribute.
        - ``.stimulus``: The ``scan.measure.stimulus`` attribute.
    """

    @property
    def interval(self) -> str:
        """Access the ``scan.measure.interval`` attribute.

        Description:
            - This attribute specifies the interval time between measurement requests.

        Usage:
            - Accessing this property will send the ``print(scan.measure.interval)`` query.
            - Setting this property to a value will send the ``scan.measure.interval = value``
              command.

        TSP Syntax:
            ```
            - scan.measure.interval = value
            - print(scan.measure.interval)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".interval"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.interval)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.interval`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @interval.setter
    def interval(self, value: Union[str, float]) -> None:
        """Access the ``scan.measure.interval`` attribute.

        Description:
            - This attribute specifies the interval time between measurement requests.

        Usage:
            - Accessing this property will send the ``print(scan.measure.interval)`` query.
            - Setting this property to a value will send the ``scan.measure.interval = value``
              command.

        TSP Syntax:
            ```
            - scan.measure.interval = value
            - print(scan.measure.interval)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".interval", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.interval = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.interval`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def stimulus(self) -> str:
        """Access the ``scan.measure.stimulus`` attribute.

        Description:
            - This attribute selects the trigger for the measurement.

        Usage:
            - Accessing this property will send the ``print(scan.measure.stimulus)`` query.
            - Setting this property to a value will send the ``scan.measure.stimulus = value``
              command.

        TSP Syntax:
            ```
            - scan.measure.stimulus = value
            - print(scan.measure.stimulus)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``scan.measure.stimulus`` attribute.

        Description:
            - This attribute selects the trigger for the measurement.

        Usage:
            - Accessing this property will send the ``print(scan.measure.stimulus)`` query.
            - Setting this property to a value will send the ``scan.measure.stimulus = value``
              command.

        TSP Syntax:
            ```
            - scan.measure.stimulus = value
            - print(scan.measure.stimulus)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class ScanChannel(BaseTSPCmd):
    """The ``scan.channel`` command tree.

    Properties and methods:
        - ``.stimulus``: The ``scan.channel.stimulus`` attribute.
    """

    @property
    def stimulus(self) -> str:
        """Access the ``scan.channel.stimulus`` attribute.

        Description:
            - This attribute determines which trigger event causes the channel action to occur.

        Usage:
            - Accessing this property will send the ``print(scan.channel.stimulus)`` query.
            - Setting this property to a value will send the ``scan.channel.stimulus = value``
              command.

        TSP Syntax:
            ```
            - scan.channel.stimulus = value
            - print(scan.channel.stimulus)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stimulus"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stimulus)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @stimulus.setter
    def stimulus(self, value: Union[str, float]) -> None:
        """Access the ``scan.channel.stimulus`` attribute.

        Description:
            - This attribute determines which trigger event causes the channel action to occur.

        Usage:
            - Accessing this property will send the ``print(scan.channel.stimulus)`` query.
            - Setting this property to a value will send the ``scan.channel.stimulus = value``
              command.

        TSP Syntax:
            ```
            - scan.channel.stimulus = value
            - print(scan.channel.stimulus)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".stimulus", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.stimulus = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stimulus`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error


class Scan(BaseTSPCmd):
    """The ``scan`` command tree.

    Properties and methods:
        - ``.add()``: The ``scan.add()`` function.
        - ``.addsinglestep()``: The ``scan.addsinglestep()`` function.
        - ``.buffer``: The ``scan.buffer`` attribute.
        - ``.bypass``: The ``scan.bypass`` attribute.
        - ``.channel``: The ``scan.channel`` command tree.
        - ``.create()``: The ``scan.create()`` function.
        - ``.export()``: The ``scan.export()`` function.
        - ``.learnlimits()``: The ``scan.learnlimits()`` function.
        - ``.list()``: The ``scan.list()`` function.
        - ``.measure``: The ``scan.measure`` command tree.
        - ``.mode``: The ``scan.mode`` attribute.
        - ``.monitor``: The ``scan.monitor`` command tree.
        - ``.restart``: The ``scan.restart`` attribute.
        - ``.scancount``: The ``scan.scancount`` attribute.
        - ``.scaninterval``: The ``scan.scaninterval`` attribute.
        - ``.start``: The ``scan.start`` command tree.
        - ``.stepcount``: The ``scan.stepcount`` attribute.
    """

    def __init__(self, device: Optional["TSPDevice"] = None, cmd_syntax: str = "scan") -> None:
        super().__init__(device, cmd_syntax)
        self._channel = ScanChannel(device, f"{self._cmd_syntax}.channel")
        self._measure = ScanMeasure(device, f"{self._cmd_syntax}.measure")
        self._monitor = ScanMonitor(device, f"{self._cmd_syntax}.monitor")
        self._start = ScanStart(device, f"{self._cmd_syntax}.start")

    @property
    def buffer(self) -> str:
        """Access the ``scan.buffer`` attribute.

        Description:
            - This attribute defines which buffer is used with the scan.

        Usage:
            - Accessing this property will send the ``print(scan.buffer)`` query.
            - Setting this property to a value will send the ``scan.buffer = value`` command.

        TSP Syntax:
            ```
            - scan.buffer = value
            - print(scan.buffer)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".buffer"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.buffer)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.buffer`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @buffer.setter
    def buffer(self, value: Union[str, float]) -> None:
        """Access the ``scan.buffer`` attribute.

        Description:
            - This attribute defines which buffer is used with the scan.

        Usage:
            - Accessing this property will send the ``print(scan.buffer)`` query.
            - Setting this property to a value will send the ``scan.buffer = value`` command.

        TSP Syntax:
            ```
            - scan.buffer = value
            - print(scan.buffer)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".buffer", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.buffer = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.buffer`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def bypass(self) -> str:
        """Access the ``scan.bypass`` attribute.

        Description:
            - This attribute indicates whether the first channel of the scan waits for the channel
              stimulus event to be satisfied before closing.

        Usage:
            - Accessing this property will send the ``print(scan.bypass)`` query.
            - Setting this property to a value will send the ``scan.bypass = value`` command.

        TSP Syntax:
            ```
            - scan.bypass = value
            - print(scan.bypass)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".bypass"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.bypass)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.bypass`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @bypass.setter
    def bypass(self, value: Union[str, float]) -> None:
        """Access the ``scan.bypass`` attribute.

        Description:
            - This attribute indicates whether the first channel of the scan waits for the channel
              stimulus event to be satisfied before closing.

        Usage:
            - Accessing this property will send the ``print(scan.bypass)`` query.
            - Setting this property to a value will send the ``scan.bypass = value`` command.

        TSP Syntax:
            ```
            - scan.bypass = value
            - print(scan.bypass)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".bypass", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.bypass = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.bypass`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def channel(self) -> ScanChannel:
        """Return the ``scan.channel`` command tree.

        Sub-properties and sub-methods:
            - ``.stimulus``: The ``scan.channel.stimulus`` attribute.
        """
        return self._channel

    @property
    def measure(self) -> ScanMeasure:
        """Return the ``scan.measure`` command tree.

        Sub-properties and sub-methods:
            - ``.interval``: The ``scan.measure.interval`` attribute.
            - ``.stimulus``: The ``scan.measure.stimulus`` attribute.
        """
        return self._measure

    @property
    def mode(self) -> str:
        """Access the ``scan.mode`` attribute.

        Description:
            - This attribute sets the relay action when the scan starts.

        Usage:
            - Accessing this property will send the ``print(scan.mode)`` query.
            - Setting this property to a value will send the ``scan.mode = value`` command.

        TSP Syntax:
            ```
            - scan.mode = value
            - print(scan.mode)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".mode"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.mode)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @mode.setter
    def mode(self, value: Union[str, float]) -> None:
        """Access the ``scan.mode`` attribute.

        Description:
            - This attribute sets the relay action when the scan starts.

        Usage:
            - Accessing this property will send the ``print(scan.mode)`` query.
            - Setting this property to a value will send the ``scan.mode = value`` command.

        TSP Syntax:
            ```
            - scan.mode = value
            - print(scan.mode)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".mode", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.mode = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.mode`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def monitor(self) -> ScanMonitor:
        """Return the ``scan.monitor`` command tree.

        Sub-properties and sub-methods:
            - ``.channel``: The ``scan.monitor.channel`` attribute.
            - ``.limit``: The ``scan.monitor.limit`` command tree.
            - ``.mode``: The ``scan.monitor.mode`` attribute.
        """
        return self._monitor

    @property
    def restart(self) -> str:
        """Access the ``scan.restart`` attribute.

        Description:
            - This function causes a scan to automatically restart if it was interrupted by a power
              failure.

        Usage:
            - Accessing this property will send the ``print(scan.restart)`` query.
            - Setting this property to a value will send the ``scan.restart = value`` command.

        TSP Syntax:
            ```
            - scan.restart = value
            - print(scan.restart)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".restart"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.restart)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.restart`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @restart.setter
    def restart(self, value: Union[str, float]) -> None:
        """Access the ``scan.restart`` attribute.

        Description:
            - This function causes a scan to automatically restart if it was interrupted by a power
              failure.

        Usage:
            - Accessing this property will send the ``print(scan.restart)`` query.
            - Setting this property to a value will send the ``scan.restart = value`` command.

        TSP Syntax:
            ```
            - scan.restart = value
            - print(scan.restart)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".restart", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.restart = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.restart`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def scancount(self) -> str:
        """Access the ``scan.scancount`` attribute.

        Description:
            - This attribute sets the number of times the scan is repeated.

        Usage:
            - Accessing this property will send the ``print(scan.scancount)`` query.
            - Setting this property to a value will send the ``scan.scancount = value`` command.

        TSP Syntax:
            ```
            - scan.scancount = value
            - print(scan.scancount)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".scancount"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.scancount)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.scancount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @scancount.setter
    def scancount(self, value: Union[str, float]) -> None:
        """Access the ``scan.scancount`` attribute.

        Description:
            - This attribute sets the number of times the scan is repeated.

        Usage:
            - Accessing this property will send the ``print(scan.scancount)`` query.
            - Setting this property to a value will send the ``scan.scancount = value`` command.

        TSP Syntax:
            ```
            - scan.scancount = value
            - print(scan.scancount)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".scancount", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.scancount = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.scancount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def scaninterval(self) -> str:
        """Access the ``scan.scaninterval`` attribute.

        Description:
            - This attribute specifies the interval time between scan starts when the scan count is
              more than one.

        Usage:
            - Accessing this property will send the ``print(scan.scaninterval)`` query.
            - Setting this property to a value will send the ``scan.scaninterval = value`` command.

        TSP Syntax:
            ```
            - scan.scaninterval = value
            - print(scan.scaninterval)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".scaninterval"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.scaninterval)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.scaninterval`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @scaninterval.setter
    def scaninterval(self, value: Union[str, float]) -> None:
        """Access the ``scan.scaninterval`` attribute.

        Description:
            - This attribute specifies the interval time between scan starts when the scan count is
              more than one.

        Usage:
            - Accessing this property will send the ``print(scan.scaninterval)`` query.
            - Setting this property to a value will send the ``scan.scaninterval = value`` command.

        TSP Syntax:
            ```
            - scan.scaninterval = value
            - print(scan.scaninterval)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_verification_enabled:  # type: ignore[union-attr]
                self._device.set_and_check(  # type: ignore[union-attr]
                    self._cmd_syntax + ".scaninterval", value
                )
            else:
                self._device.write(  # type: ignore[union-attr]
                    f"{self._cmd_syntax}.scaninterval = {value}"
                )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.scaninterval`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    @property
    def start(self) -> ScanStart:
        """Return the ``scan.start`` command tree.

        Sub-properties and sub-methods:
            - ``.stimulus``: The ``scan.start.stimulus`` attribute.
        """
        return self._start

    @property
    def stepcount(self) -> str:
        """Access the ``scan.stepcount`` attribute.

        Description:
            - This attribute returns the number of steps in the present scan.

        Usage:
            - Accessing this property will send the ``print(scan.stepcount)`` query.

        TSP Syntax:
            ```
            - print(scan.stepcount)
            ```

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            if self._device.command_syntax_enabled:  # type: ignore[union-attr]
                return self._cmd_syntax + ".stepcount"
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.stepcount)"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to access the ``{self._cmd_syntax}.stepcount`` attribute."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def add(
        self, channel_list: str, config_list: Optional[str] = None, index: Optional[int] = None
    ) -> None:
        """Run the ``scan.add()`` function.

        Description:
            - This function adds channels to the scan list.

        TSP Syntax:
            ```
            - scan.add()
            ```

        Args:
            channel_list: List of channels to add, in the order in which they should occur in the
                scan.
            config_list (optional): A string that defines the source or measure configuration list
                to recall.
            index (optional): The index in the configuration list to recall; default is 1.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    f'"{config_list}"' if config_list is not None else None,
                    index,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.add({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.add()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def addsinglestep(
        self, channel_list: str, config_list: Optional[str] = None, index: Optional[int] = None
    ) -> None:
        """Run the ``scan.addsinglestep()`` function.

        Description:
            - This function allows you to include multiple channels in a single scan step.

        TSP Syntax:
            ```
            - scan.addsinglestep()
            ```

        Args:
            channel_list: List of channels to add, in the order in which they should occur in the
                scan.
            config_list (optional): A string that defines the source or measure configuration list
                to recall.
            index (optional): The index in the configuration list to recall; default is 1.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"',
                    config_list,
                    index,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.addsinglestep({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.addsinglestep()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def create(
        self,
        channel_list: Optional[str] = None,
        config_list: Optional[str] = None,
        index: Optional[int] = None,
    ) -> None:
        """Run the ``scan.create()`` function.

        Description:
            - This function deletes the existing scan list and creates a new list of channels and
              channel patterns and channel patterns to scan.

        TSP Syntax:
            ```
            - scan.create()
            ```

        Args:
            channel_list (optional): String specifying channels to add to the new scan list.
            config_list (optional): A string that defines the source or measure configuration list
                to recall.
            index (optional): The index in the configuration list to recall; default is 1.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    f'"{channel_list}"' if channel_list is not None else None,
                    config_list,
                    index,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.create({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.create()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def export(self, filename: str, when: str, what: Optional[str] = None) -> None:
        """Run the ``scan.export()`` function.

        Description:
            - This command stores data from a scan to a file on a USB flash drive.

        TSP Syntax:
            ```
            - scan.export()
            ```

        Args:
            filename: The name of the file to be created on the USB flash drive.
            when: When to write the data to the file.
            what (optional): Which data to include; see Details for options.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    filename,
                    when,
                    what,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.export({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.export()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def learnlimits(self, window: str, iterations: Optional[str] = None) -> None:
        """Run the ``scan.learnlimits()`` function.

        Description:
            - This function calculates alarm limits based on the present configuration of the
              system.

        TSP Syntax:
            ```
            - scan.learnlimits()
            ```

        Args:
            window: Percentage of deviation from the measurement that is within limits.
            iterations (optional): Number of times to run the scan to set limits.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            function_args = ", ".join(
                str(x)
                for x in (
                    window,
                    iterations,
                )
                if x is not None
            )
            self._device.write(  # type: ignore[union-attr]
                f"{self._cmd_syntax}.learnlimits({function_args})"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.learnlimits()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error

    def list(self) -> str:
        """Run the ``scan.list()`` function.

        Description:
            - This function queries the active scan list.

        TSP Syntax:
            ```
            - scan.list()
            ```

        Returns:
            The result of the function call.

        Raises:
            tm_devices.commands.NoDeviceProvidedError: Indicates that no device connection exists.
        """
        try:
            return self._device.query(  # type: ignore[union-attr]
                f"print({self._cmd_syntax}.list())"
            )
        except AttributeError as error:
            msg = f"No TSPDevice object was provided, unable to run the ``{self._cmd_syntax}.list()`` function."  # noqa: E501
            raise NoDeviceProvidedError(msg) from error
